<?php

namespace Drupal\cfrapi\CfrCodegenHelper;

use Donquixote\CallbackReflection\CodegenHelper\CodegenHelperBase;
use Drupal\cfrapi\Exception\ConfToValueException;
use Drupal\cfrapi\Exception\PhpGenerationNotSupportedException;

class CfrCodegenHelper extends CodegenHelperBase implements CfrCodegenHelperInterface {

  /**
   * @var array
   */
  private $problems = [];

  /**
   * {@inheritdoc}
   */
  public function recursionDetected($conf, $message) {

    $this->problems[] = t('Recursion detected.');

    $php = ''
      . "\n" . '$conf = ' . var_export($conf, TRUE) . ';'
      . "\n" . 'throw new \\' . PhpGenerationNotSupportedException::class . '(' . var_export($message, TRUE) . ');';

    return '// @todo Fix the configuration to prevent recursion.'
      . "\n" . 'call_user_func('
      . "\n" . 'function(){' . $php
      . "\n" . '})';
  }

  /**
   * {@inheritdoc}
   */
  public function incompatibleConfiguration($conf, $message) {

    $this->problems[] = t('Incompatible configuration.');

    $php = ''
      . "\n" . '$conf = ' . var_export($conf, TRUE) . ';'
      . "\n" . 'throw new \\' . ConfToValueException::class . '(' . var_export($message, TRUE) . ');';

    return '// @todo Fix the configuration, before exporting this to code!'
      . "\n" . 'call_user_func('
      . "\n" . 'function(){' . $php
      . "\n" . '})';
  }

  /**
   * {@inheritdoc}
   */
  public function notSupported($object, $conf, $message) {

    $this->problems[] = t('PHP generation not supported.');

    $php = ''
      . "\n" . '$class = ' . var_export(\get_class($object), TRUE) . ';'
      . "\n" . '$conf = ' . var_export($conf, TRUE) . ';'
      . "\n" . 'throw new \\' . PhpGenerationNotSupportedException::class . '(' . var_export($message, TRUE) . ');';

    return '// @todo Fix the generated code manually.'
      . "\n" . 'call_user_func('
      . "\n" . 'function(){' . $php
      . "\n" . '})';
  }

  /**
   * {@inheritdoc}
   */
  protected function addProblem($message) {
    $this->problems[] = t($message);
  }

}
