<?php

namespace Drupal\cfrapi\Configurator\Optional;

use Drupal\cfrapi\Configurator\ConfiguratorInterface;

/**
 * Configurator where some configuration values are considered to be 'empty'.
 *
 * This concept is currently only used for an outdated version of the sequence
 * configurator, where "non-empty" items cause the sequence to grow, and "empty"
 * items are removed.
 * See https://www.drupal.org/project/cfr/issues/3165152
 *
 * @see \Drupal\cfrapi\Configurator\Sequence\Configurator_Sequence
 *
 * @todo Deprecate and remove, including subclasses, see #3165150.
 */
interface OptionalConfiguratorInterface extends ConfiguratorInterface {

  /**
   * Gets an object to detect and create 'empty' configuration values.
   *
   * @return \Drupal\cfrapi\ConfEmptyness\ConfEmptynessInterface|null
   *   An emptyness object, or
   *   NULL, if the configurator is in fact required and thus no valid conf
   *   counts as empty.
   */
  public function getEmptyness();

}
