<?php

use Drupal\cfrplugin\Hub\CfrPluginHub;
use Drupal\cfrreflection\Util\StringUtil;
use Drupal\themekit\T;

/**
 * Page callback for the overview page at 'admin/reports/cfrplugin'.
 *
 * @return array
 *   Page content render element.
 */
function _cfrplugin_report_overview_page() {

  $services = CfrPluginHub::getContainer();
  $definitions = $services->definitionsByTypeAndId->getDefinitionsByTypeAndId();

  $rows = [];
  $rows_grouped = [];
  foreach ($definitions as $interface => $interface_definitions) {

    $label = StringUtil::interfaceGenerateLabel($interface);
    $count = t('@n plugin definitions', ['@n' => count($interface_definitions)]);
    $interface_shortname = StringUtil::classGetShortname($interface);
    $interface_basename = preg_replace('@Interface$@', '', $interface_shortname);
    $path = 'admin/reports/cfrplugin/' . _cfrplugin_interface_slug($interface);

    $row = [];
    $row[] = $label;
    // Link to a list of plugins for the interface.
    $row[] = l($count, $path);
    // Link to a demo form.
    $row[] = l(t('Demo: @name', ['@name' => $interface_basename . '*']), $path . '/demo');
    // Link to a page showing the interface declaration.
    $row[] = l('<code>' . $interface . '</code>', $path . '/code', ['html' => TRUE]);

    $fragments = explode('\\', $interface);
    if (1
      && 'Drupal' === $fragments[0]
      && isset($fragments[2])
      && module_exists($fragments[1])
    ) {
      $rows_grouped[$fragments[1]][] = $row;
    }
    else {
      $rows[] = $row;
    }
  }

  $modules_info = system_get_info('module_enabled');

  foreach ($rows_grouped as $module => $module_rows) {

    $module_label = isset($modules_info[$module])
      ? $modules_info[$module]['name']
      : $module;

    $rows[] = [
      [
        'colspan' => 4,
        'data' => '<h3>' . $module_label . '</h3>',
      ],
    ];

    foreach ($module_rows as $row) {
      $rows[] = $row;
    }
  }

  return [
    '#header' => [
      t('Human name'),
      t('List of plugins'),
      t('Demo'),
      t('Interface declaration'),
    ],
    '#theme' => T::th('theme_table'),
    '#rows' => $rows,
  ];
}
